"""
    Sky

    The sky-ball

    Pyweek 5 Scavengers team entry Sep 2007

    GPL v3 or later
"""

from data import filepath

from tdlib import backdrop, texture, lighting
from OpenGL.GL import *
from OpenGL.GLU import *

class SkyBall(backdrop.TextureBackdrop):
    _default_geom = {'radius':500.0}
    tex = None

    def __init__(self,name='',**kwd):
        self.quadric = q = gluNewQuadric()
        super(SkyBall,self).__init__(name,**kwd)
        gluQuadricDrawStyle(q, GLU_FILL)
        gluQuadricOrientation(q,GLU_INSIDE)
        gluQuadricNormals(q,GLU_SMOOTH)

    def prepare(self):
        texname = self.getstyle('texture')
        if texname:
            self.tex = texture.get_texture(filepath(texname))
            self._has_transparent = self.tex.transparent
            gluQuadricTexture(self.quadric, GL_TRUE)
        else:
            self.tex = None
            self._has_transparent = False
            gluQuadricTexture(self.quadric,GL_FALSE)
        self.colour = self._style['colour']
        self._has_transparent = self._has_transparent or  (self.colour[3] < 1.0)

    def setup_style(self):
        glPushAttrib(GL_LIGHTING_BIT)
        glDisable(GL_LIGHTING)
        if self.tex:
            glEnable(GL_TEXTURE_2D)
            self.tex.bind()
        glColor4fv(self.colour)
            
    def setdown_style(self):
        glPopAttrib()
        
    def render(self,mode):
        if mode == "OPAQUE":
            glClear(backdrop.BUFFERBITS)  # only clear on first pass
        elif mode == "PICK":
            return  # no need to pick
        glDepthMask(GL_FALSE)
        gluSphere(self.quadric, self.getgeom("radius"), 16, 16)
        if mode != "TRANSPARENT":
            glDepthMask(GL_TRUE)
        
